
import arcade
import math
import random

# parametros para uma arvore frondosa
C_COMPR = 2.7
C_ESP = 0.12
THETA_F = 20
C_M = 10
C_S = 30
CHANCE_TERM = 5
N_FOLHAGEM = 3
LADOS_FOLHA = 3
MAX_ORDEM = 16
COR_FOLHAS = [
    arcade.color.GO_GREEN,
    arcade.color.FOREST_GREEN,
    arcade.color.DARK_GREEN,
    arcade.color.DARK_SPRING_GREEN]
    
def cria_ramos(xA, yA, ang, ordem):
    global ramos

    compr = C_COMPR*ordem*(1 + random.random())
    esp = C_ESP*math.pow(ordem,2)
  
    ang_rad = ang*math.pi/180
    xB = xA + compr*math.cos(ang_rad)
    yB = yA + compr*math.sin(ang_rad)

    ramos.append([xA, yA,
                  xB, yB,
                  ordem,
                  esp, compr])
    
    if ordem > 1:
        # a principio fork
        # com main e secondary iguais
        ordM = ordem - 1
        ordS = ordem - 1

        d_angM = THETA_F
        d_angS = -THETA_F

        # chance de bifurcar para um ramo terminal
        term = random.randint(1, CHANCE_TERM)
        if ordM > 1 and term == 1:
            ordS = 1
            d_angM = C_M*ordS/(ordM - 1)
            d_angS = -C_S*(ordM - ordS)/(ordM - 1)

        angM = ang + d_angM
        angS = ang + d_angS

        cria_ramos(xB, yB,
                   angM, ordM)        
        cria_ramos(xB, yB,
                   angS, ordS)

def desenha_ramos(arcade):
    global ramos
    for ramo in ramos:
        xA, yA = ramo[0], ramo[1]
        xB, yB = ramo[2], ramo[3]
        ordem = ramo[4]
        esp = ramo[5]

        dc = (MAX_ORDEM - ordem)*5
        # cor inicial eh a SEPIA e vai clareando
        r = min(112 + dc, 255)
        g = min( 66 + dc, 255)
        b = min( 20 + dc, 255)
     
        arcade.draw_line(xA, yA, xB, yB,
            (r, g, b), esp)

def cria_folhagem():
    global ramos
    global folhagem

    ang_central_rad = math.pi*360/LADOS_FOLHA/180
    for ramo in ramos:
        xB, yB = ramo[2], ramo[3]
        ordem = ramo[4]        
        cor = random.randint(0, len(COR_FOLHAS) - 1)
 
        off = (MAX_ORDEM - ordem)
        raio = max(ordem/4,2)
        for i in range(N_FOLHAGEM):
            # deslocamento da folhagem
            dx = random.randint(-off//2,off//2)
            dy = random.randint(-off//2,off//2)
            # orientacao do poligono
            ang_rad = math.pi*\
                random.randint(0,12)*30/180
            # calcula vertices do poligono
            pts = []
            for j in range(LADOS_FOLHA):
                x = xB + dx +\
                    raio*math.cos(ang_rad)
                y = yB + dy +\
                    raio*math.sin(ang_rad)
                ang_rad += ang_central_rad
                pts.append((x,y))
            folhagem.append((pts, cor))

def desenha_folhagem(arcade):
    global folhagem
    for folha in folhagem:
        arcade.draw_polygon_filled(folha[0],
            COR_FOLHAS[folha[1]])

def sort_folhas(e):
    # sort pelo indice da cor, menor o indice
    # mais clara, mais para frente
    return e[1]

# tamanho da janela que vamos usar
LARG_TELA = 800
ALT_TELA = 600

# para poder repetir os resultados
random.seed(0)

# calcula os vertices dos ramos da arvoce
ramos = []
folhagem = []
cria_ramos(LARG_TELA/2, 0, 90, MAX_ORDEM)
cria_folhagem()
# z-order
folhagem.sort(reverse = True, key = sort_folhas)

# nome da janela
TIT_TELA = 'Arvore'
# cria janela com dimensoes
arcade.open_window(LARG_TELA, ALT_TELA, TIT_TELA)
# define cor de fundo da tela
arcade.set_background_color(arcade.color.WHITE)
# apaga a tela e comeca a desenhar
arcade.start_render()

desenha_ramos(arcade)
desenha_folhagem(arcade)

# termina de desenhar e mostra a tela
arcade.finish_render()
# espera usuario fechar a janela
arcade.run()

        
    
