"""
Arte proveniente do autor :)
"""

import arcade
import os

# velocidade do aviao e escala dos sprites
V_X_AVIAO = 4
V_Y_AVIAO = 4
ESCALA = 0.6

# tamanho e titulo da janela que vamos usar
LARG_TELA = 500
ALT_TELA = 200
TIT_TELA = 'Colisao'

class AviaoSprite(arcade.Sprite):
    def __init__(self, arq, bbox, hitbox = 'None'):
        super().__init__(arq, ESCALA,
                         hit_box_algorithm = hitbox)
        # coordenadas do espaco aereo
        self.ea_esq = bbox[0]
        self.ea_inf = bbox[1]
        self.ea_dir = bbox[2]
        self.ea_sup = bbox[3]
        # coordenada inicial do sprite
        self.center_x = (self.ea_dir + self.ea_esq)/2
        self.center_y = (self.ea_sup + self.ea_inf)/2
        
    def update(self):
        if self.left < self.ea_esq:
            self.left = self.ea_esq
        elif self.bottom < self.ea_inf:
            self.bottom = self.ea_inf
        elif self.right > self.ea_dir:
            self.right = self.ea_dir
        elif self.top > self.ea_sup:
            self.top = self.ea_sup
        # chama a superclasse
        super().update()

class RaioSprite(arcade.Sprite):
    def __init__(self, arq, bbox, hitbox = 'None'):
        super().__init__(arq, ESCALA,
                         hit_box_algorithm = hitbox)
        self.ea_esq = bbox[0]
        self.ea_inf = bbox[1]
        self.ea_dir = bbox[2]
        self.ea_sup = bbox[3]
        # coordenada inicial do sprite
        self.center_x = (self.ea_dir + self.ea_esq)/2
        # nao pode ser top devido ao hit_box
        self.center_y = self.ea_sup -\
                        (self.ea_sup - self.ea_inf)/4       

class MeuJogo(arcade.Window):
    def __init__(self):
        super().__init__(LARG_TELA, ALT_TELA,
            TIT_TELA)           
        # muda o diretorio corrente para o mesmo
        # diretorio que esta o fonte .py
        file_path = os.path.dirname(
            os.path.abspath(__file__))
        os.chdir(file_path)
    
        arcade.set_background_color(
            arcade.color.WHITE)
        # esconde o cursor do mouse
        self.set_mouse_visible(False)

        # desenha os hit_boxes?
        self.hit_boxes = False
        # algoritmos de colisao
        hit_box = ['None', 'Simple', 'Detailed']
        self.aviao_list = arcade.SpriteList()
        self.raio_list = arcade.SpriteList()
        for i in range(3):
            x0 = i*LARG_TELA/3
            y0 = 0
            x1 = (i+1)*LARG_TELA/3
            y1 = ALT_TELA
            bbox = [x0,y0,x1,y1]

            arq = 'sprites_exemplos/aviao_transp.png'
            aviao = AviaoSprite(arq,bbox,hit_box[i])
            self.aviao_list.append(aviao)

            arq = 'sprites_exemplos/raio_transp.png'
            raio = RaioSprite(arq,bbox,hit_box[i])
            self.raio_list.append(raio)

    def on_update(self, delta_time):
        for aviao in self.aviao_list:
            hit_list =\
                arcade.\
                check_for_collision_with_list(
                aviao, self.raio_list)
            if len(hit_list) > 0:
                aviao.alpha = 127
            else:
                aviao.alpha = 255
        self.aviao_list.update()
        self.raio_list.update()
    
    def on_draw(self):
        arcade.start_render()
        self.aviao_list.draw()
        self.raio_list.draw()
        if self.hit_boxes is True:
            for aviao in self.aviao_list:
                aviao.draw_hit_box(arcade.color.RED,
                                    2)
            for raio in self.raio_list:
                raio.draw_hit_box(arcade.color.RED,
                                  2)

    def on_key_press(self, key, modifiers):
        change_x = 0
        change_y = 0
     
        if key == arcade.key.LEFT:
            change_x = -V_X_AVIAO
        elif key == arcade.key.RIGHT:
            change_x = V_X_AVIAO
        elif key == arcade.key.UP:
            change_y = V_Y_AVIAO
        elif key == arcade.key.DOWN:
            change_y = -V_Y_AVIAO
        elif key == arcade.key.H:
            self.hit_boxes = not self.hit_boxes

        if change_x != 0 or change_y != 0:
            for aviao in self.aviao_list:
                aviao.change_x = change_x
                aviao.change_y = change_y

    def on_key_release(self, key, modifiers):
        if key == arcade.key.LEFT or\
            key == arcade.key.RIGHT or\
            key == arcade.key.UP or\
            key == arcade.key.DOWN:
            for aviao in self.aviao_list:
                aviao.change_x = 0
                aviao.change_y = 0

def main():
    window = MeuJogo()
    arcade.run()

if __name__ == "__main__":
    main()
