
# modulo 04, ex 04
import arcade
import math

# raio dos numeros (horas)
R_NUM = 80
# tamanho da fonte
F_NUM = 30
# cor dos numeros
C_NUM = arcade.color.WHITE

# raio dos marcadores
R_MAR = 95
# espessura dos marcadores
E1_MAR = 4
E2_MAR = 8
# cor dos mostradores
C1_MAR = arcade.color.WHITE
C2_MAR = arcade.color.RED

# raio dos ponteiros
R_HOR = 60
R_SEG = 90
R_MIN = 90
# espessura dos ponteiros
E_HOR = 15
E_MIN = 10
E_SEG = 1
# cor do ponteiros
C_HOR = arcade.color.WHITE
C_MIN = arcade.color.WHITE
C_SEG = arcade.color.RED

# FPS do relogio
FPS_RELOGIO = 5

# tamanho e titulo da janela que vamos usar
LARG_TELA = 600
ALT_TELA = 600
TIT_TELA = 'Relogio'

class Relogio(arcade.Window):
    def __init__(self):
        super().__init__(LARG_TELA, ALT_TELA,
            TIT_TELA)
        arcade.set_background_color(
            arcade.color.BLACK)
        # FPS
        self.set_update_rate(1/FPS_RELOGIO)
        # incremento do ponteiro dos segundos [s]
        self.delta_seg = 1/FPS_RELOGIO

        # meio da tela
        self.xc = LARG_TELA/2
        self.yc = ALT_TELA/2

        # arrays com as principais constantes
        # raio dos ponteiros
        self.raio = []
        self.raio.append(LARG_TELA/2*R_HOR/100)
        self.raio.append(LARG_TELA/2*R_MIN/100)
        self.raio.append(LARG_TELA/2*R_SEG/100)

        # cores
        self.cor = []
        self.cor.append(C_HOR)
        self.cor.append(C_MIN)
        self.cor.append(C_SEG)

        # espessura
        self.esp = []
        self.esp.append(E_HOR)
        self.esp.append(E_MIN)
        self.esp.append(E_SEG)

        # hora inicial
        self.t_hor = 9.0
        self.t_min = 15.0
        self.t_seg = 30.0

        # numeros do mostrador
        self.numeros = []
        # marcadores e ponteiros
        self.marcadores = arcade.ShapeElementList()
        self.ponteiros = arcade.ShapeElementList()
  
    def setup_marcadores(self):
        raio = LARG_TELA/2*R_MAR/100
        cor = C1_MAR
        esp = E1_MAR
        for i in range(60):
            x = self.xc + raio*math.cos(math.pi*i*6/180)
            y = self.yc + raio*math.sin(math.pi*i*6/180)
            shape = arcade.create_rectangle_filled(x,y,
                esp, esp,
                cor)
            self.marcadores.append(shape)

        cor = C2_MAR
        esp = E2_MAR
        for i in range(12):
            x = self.xc + raio*math.cos(math.pi*i*30/180)
            y = self.yc + raio*math.sin(math.pi*i*30/180)
            shape = arcade.create_rectangle_filled(x,y,
                esp, esp,
                cor)
            self.marcadores.append(shape)

    def setup_numeros(self):
        raio = LARG_TELA/2*R_NUM/100
        for i in range(12):
            hora = i
            # 12 horas e nao 0 hora
            if i == 0:
                hora = 12
            x = self.xc + raio*math.cos(math.pi*(90-i*30)/180)
            y = self.yc + raio*math.sin(math.pi*(90-i*30)/180)
            num = arcade.Text(
                text=str(hora),
                start_x=x, start_y=y,
                color=C_NUM,
                font_name ='Courier New',
                font_size=F_NUM,
                anchor_x='center',
                anchor_y='center')
            self.numeros.append(num)

    def calcula_ponteiros(self):
        ang_drg = []
        ang_drg.append(90 - 30*(
            self.t_hor +\
            self.t_min/60 +\
            self.t_seg/3600))
        ang_drg.append(90 - 6*(
            self.t_min +\
            self.t_seg/60))
        ang_drg.append(90 - 6*self.t_seg)

        ang_rad = [math.pi*ang/180
            for ang in ang_drg]

        self.ponteiros = arcade.ShapeElementList()       
        for i in range(3):
            x = self.xc + self.raio[i]*math.cos(ang_rad[i])
            y = self.yc + self.raio[i]*math.sin(ang_rad[i])
            shape = arcade.create_line(
                self.xc, self.yc,
                x, y,
                self.cor[i], self.esp[i])
            self.ponteiros.append(shape)

    def on_update(self, delta_time):
        self.t_seg += self.delta_seg
        if self.t_seg >= 60:
            self.t_seg = 0
            self.t_min += 1

        if self.t_min >= 60:
            self.t_min = 0
            self.t_hor += 1

        if self.t_hor >= 12:
            self.t_hor = 0

        self.calcula_ponteiros()

    def on_draw(self):
        self.clear()
        for numero in self.numeros:
            numero.draw()
        self.marcadores.draw()
        self.ponteiros.draw()
       
    def on_key_press(self, key, modifiers):
        if key == arcade.key.RIGHT:
            self.t_min += 1
        elif key == arcade.key.LEFT:
            self.t_min -= 1
        elif key == arcade.key.UP:
            self.t_min += 5 
        elif key == arcade.key.DOWN:
            self.t_min -= 5
        elif key == arcade.key.Z:
            self.t_seg = 0

def main():
    window = Relogio()
    window.setup_numeros()
    window.setup_marcadores()
    arcade.run()

if __name__ == "__main__":
    main()
