import arcade
import random
import math

# velocidade e dimensao maxima das figuras
V_X = 5
V_Y = 5
MAX_DIM = 60

# tamanho e titulo da janela que vamos usar
LARG_TELA = 400
ALT_TELA = 300
TIT_TELA = 'Figuras'

# rabisco
N_PONTOS = 10
ESP_RABISCO = 3

class Figura:
    def __init__(self):
        # coordenada inicial da entidade
        self.x = random.randint(0, LARG_TELA)
        self.y = random.randint(0, ALT_TELA)
        # velocidade
        self.v_x = random.randint(1,V_X)
        self.v_y = random.randint(1,V_Y)        
        # dimensao
        self.dim = random.randint(1,MAX_DIM)
        # cor
        r = random.randint(0,255)
        g = random.randint(0,255)
        b = random.randint(0,255)
        self.cor = (r,g,b)

    def move(self):
        self.x += self.v_x
        self.y += self.v_y
        
        if self.x < 0 or self.x > LARG_TELA:
            self.v_x = -self.v_x
        if self.y < 0 or self.y > ALT_TELA:
            self.v_y = -self.v_y  

class Circulo(Figura):
    def __init__(self):
        super().__init__()

    def desenha(self, arcade):
        arcade.draw_circle_filled(
            self.x, self.y,
            self.dim/2, self.cor)

class Retangulo(Figura):
    def __init__(self):
        super().__init__()
        # retangulo na horizontal ou vertical
        if random.randint(0,1) == 0:
            self.dim_x = self.dim
            self.dim_y = self.dim * 2
        else:
            self.dim_x = self.dim * 2
            self.dim_y = self.dim

    def desenha(self, arcade):
        arcade.draw_rectangle_filled(
            self.x, self.y,
            self.dim_x, self.dim_y,
            self.cor)

class Quadrado(Retangulo):
    def __init__(self):
        super().__init__()
        self.dim_x = self.dim_y

class Triangulo(Figura):
    def __init__(self):
        super().__init__()
        # angulo interno 60 graus
        self.h = self.dim*math.sqrt(3)/2

    def desenha(self, arcade):
        # triangulo equilatero
        xA = self.x - self.dim/2
        yA = self.y
        xB = self.x + self.dim/2
        yB = self.y
        xC = self.x
        yC = yA + self.h
        arcade.draw_triangle_filled(
            xA, yA, xB, yB, xC, yC,
            self.cor)

class Rabisco(Figura):
    def __init__(self):
        super().__init__()
        self.dxdy = []
        for i in range(N_PONTOS):
            self.dxdy.append([
                random.randint(-self.dim//2,\
                    self.dim//2),
                random.randint(-self.dim//2,\
                    self.dim//2)
                ])

    def desenha(self, arcade):
        pts = []
        for dx, dy in self.dxdy:
            pts.append([self.x + dx, self.y + dy])
        arcade.draw_line_strip(pts, self.cor,\
            ESP_RABISCO)

class MeuJogo(arcade.Window):
    def __init__(self):
        super().__init__(LARG_TELA, ALT_TELA,
            TIT_TELA)
        arcade.set_background_color(
            arcade.color.WHITE)
        self.figs = [Circulo, Quadrado,\
                Triangulo, Retangulo,\
                Rabisco]
        self.n_figs = len(self.figs)
        self.fig_list = []
    
    def on_update(self, delta_time):
        for fig in self.fig_list:
            fig.move()
    
    def on_draw(self):
        arcade.start_render()
        for fig in self.fig_list:
            fig.desenha(arcade)

    def on_key_press(self, key, modifiers):
        if key == arcade.key.SPACE:
            tipo = len(self.fig_list) % self.n_figs
            fig = self.figs[tipo]()            
            self.fig_list.append(fig)

def main():
    window = MeuJogo()
    arcade.run()

if __name__ == "__main__":
    main()
