"""
Jogo de Naves, livremente inspirado
nos classicos Space Invaders e Galaxian
Arte proveniente do http://kenney.nl
https://kenney.nl/assets/space-shooter-redux
"""

import random
import arcade
import math
import os

# constantes
ESCALA_ESTRELA = 0.5
D_ALPHA_ESTRELA = 3
V_Y_ESTRELA = 3
QTD_ESTRELAS = 100

ESCALA_NAVE = 0.5
V_X_NAVE = 5

ESCALA_VIDA = 0.8

QTD_VIDAS = 3
DT_REVIVE = 200

ESCALA_FASE_P = 0.8
ESCALA_FASE_G = 0.9

V_Y_MISSIL = 9

ESCALA_INIMIGO = 0.4
LINS_INIMIGOS = 5
COLS_INIMIGOS = 7

V_X_INIMIGO = 2
V_Y_INIMIGO = 4
A_X_INIMIGO = 0.1

V_Y_INIMISSIL = 5
P_INIMISSIL = 500

ESCALA_UFO = 0.4
V_X_UFO = 5
P_UFO = 1000
DT_UFO = 200

LARG_TELA = 800
ALT_TELA = 600
MARGEM_Y_TELA = 25
TIT_TELA = 'Invaxians'

class EstrelaSprite(arcade.Sprite):
    def __init__(self, filename):
        # calcula escala para mostrar estrelas
        # de tamanhos diferentes
        escala = random.uniform(0.6, 1.0)
        escala *= ESCALA_ESTRELA
        # chama o construtor da superclasse
        super().__init__(filename, escala)
        # posicao da estrela
        self.center_x = random.randint(0,LARG_TELA)
        self.center_y = random.randint(0, ALT_TELA)
        # brilho inicial (transparencia) da estrela
        self.alpha = random.randint(0, 255)
        # taxa de variacao  do brilho
        self.d_alpha = random.randint(1,
            D_ALPHA_ESTRELA)
        # taxa de variacao da posicao Y, i.e,
        # velocidade em Y
        self.change_y = -random.randint(1,
            V_Y_ESTRELA)

    def update(self):
        # ajusta o alpha para a estrela piscar
        alpha = self.alpha + self.d_alpha
        if alpha < 0 or alpha > 255:
            self.d_alpha = -self.d_alpha
            alpha += self.d_alpha
        self.alpha = alpha
        # se saiu da tela, volta para tela
        # e muda velocidade Y
        if self.center_y < 0:
            self.center_y = ALT_TELA
            self.change_y = -random.randint(1,
                V_Y_ESTRELA)
        # chama a superclasse
        super().update()

class NaveSprite(arcade.Sprite):
    def __init__(self, filename):
        # chama o construtor da superclasse
        super().__init__(filename, ESCALA_NAVE)
        self.center_x = LARG_TELA / 2
        self.bottom = MARGEM_Y_TELA

    def update(self):
        # nao deixa sair da tela
        if self.left < 0:
            self.left = 0
        elif self.right > LARG_TELA - 1:
            self.right = LARG_TELA - 1
        # chama a superclasse
        super().update()

class MissilSprite(arcade.Sprite):
    def __init__(self, filename, nave):
        # chama o construtor da superclasse
        super().__init__(filename, ESCALA_NAVE)
        self.change_y = V_Y_MISSIL
        self.center_x = nave.center_x
        self.bottom = nave.top

class InimigoSprite(arcade.Sprite):
    def __init__(self, filename, col, lin):
        # chama o construtor da superclasse
        super().__init__(filename, ESCALA_INIMIGO)
        i = col - COLS_INIMIGOS/2
        x = LARG_TELA/2 + 1.2*i*self.width
        y = (ALT_TELA - MARGEM_Y_TELA)\
            - 1.2*lin*self.height
        self.center_x = x
        self.top = y         
        self.change_x = V_X_INIMIGO

class InimissilSprite(arcade.Sprite):
    def __init__(self, filename, nave):
        # chama o construtor da superclasse
        super().__init__(filename, ESCALA_INIMIGO)
        self.change_y = -V_Y_INIMISSIL
        self.center_x = nave.center_x
        self.top = nave.bottom
        self.angle = 180

class MeuJogo(arcade.Window):    
    def __init__(self):
        # inicializa a superclasse
        super().__init__(
            LARG_TELA, ALT_TELA,
            TIT_TELA)
        # muda o diretorio corrente para o mesmo
        # diretorio que esta o fonte .py
        file_path = os.path.dirname(
            os.path.abspath(__file__))
        os.chdir(file_path)

        # lista dos sprites
        self.estrela_list = None
        self.nave_list = None
        self.vida_lista = None
        self.fase_list = None
        self.missil_list = None
        self.inimigo_list = None
        self.inimissil_list = None
        self.ufo_list = None

        # placar, fim de jogo?, revivendo,
        # fase, ufo, pausado?
        self.placar = 0
        self.game_over = False
        self.revive = 0
        self.fase = 1
        self.bonus_ufo = 0
        self.pausado = False

        # nome dos sprites dos inimigos
        self.tipo_inimigo = []
        for i in range(5):
            self.tipo_inimigo.append(
                f'enemyGreen{i+1}.png')

        # esconde o cursor do mouse
        self.set_mouse_visible(False)

        # printa o fundo da tela
        arcade.set_background_color(
            arcade.color.MIDNIGHT_BLUE)

    def incia_bg(self):
        # lista dos sprites
        self.estrela_list = arcade.SpriteList()
        # cria as estrelas
        path_sprites = 'spaceshooter/PNG/Effects/' 
        for i in range(QTD_ESTRELAS):
            estr = EstrelaSprite(
                path_sprites + 'star1.png')
            # adiciona na lista de sprites
            self.estrela_list.append(estr)

    def inicia_jogo(self):
        # lista dos sprites
        self.nave_list = arcade.SpriteList()
        self.vida_list = arcade.SpriteList()
        self.fase_list = arcade.SpriteList()
        self.missil_list = arcade.SpriteList()
        self.inimigo_list = arcade.SpriteList()
        self.inimissil_list = arcade.SpriteList()
        self.ufo_list = arcade.SpriteList()

        # cria a nave e adiciona na lista 
        path_sprites = 'spaceshooter/PNG/' 
        nave = NaveSprite(
            path_sprites + 'playerShip2_red.png')
        self.nave_list.append(nave)

        # cria vidas
        path_sprites = 'spaceshooter/PNG/UI/' 
        for i in range(QTD_VIDAS):
            vida = arcade.Sprite(
                path_sprites + 'playerLife2_red.png',
                ESCALA_VIDA)
            vida.left = 1.2*i*vida.width
            vida.bottom = 0
            self.vida_list.append(vida)

        # cria fases
        path_sprites = 'spaceshooter/PNG/Power-ups/' 
        n_fase_g = self.fase // 5
        n_fase_p = self.fase % 5
        j = 0
        for i in range(n_fase_g):
            fase = arcade.Sprite(
                path_sprites + 'star_gold.png',
                ESCALA_FASE_G)
            fase.right = LARG_TELA - 1.2*j*fase.width
            fase.bottom = 0
            self.fase_list.append(fase)
            j += 1
        for i in range(n_fase_p):
            fase = arcade.Sprite(
                path_sprites + 'star_bronze.png',
                ESCALA_FASE_P)
            fase.right = LARG_TELA - 1.2*j*fase.width
            fase.bottom = 0
            self.fase_list.append(fase)
            j += 1
        
        # cria inimigos
        path_sprites = 'spaceshooter/PNG/Enemies/' 
        for j in range(LINS_INIMIGOS):
            for i in range(COLS_INIMIGOS):
                tipo = j % len(self.tipo_inimigo)
                inimigo = InimigoSprite(
                    path_sprites
                    + self.tipo_inimigo[tipo],
                    i, j)
                self.inimigo_list.append(inimigo)

        self.revive = 0
        self.game_over = False
        self.bonus_ufo = 0

    def on_draw(self):
        arcade.start_render()
        
        # desenha sprites
        self.estrela_list.draw()
        self.nave_list.draw()
        self.vida_list.draw()
        self.fase_list.draw()
        self.missil_list.draw()
        self.inimigo_list.draw()
        self.inimissil_list.draw()
        self.ufo_list.draw()

        # atualiza placar
        arcade.draw_text(
            text=str(self.placar),
            start_x=5, start_y=ALT_TELA - 5,
            font_name='Courier New', font_size=20,
            bold=True,
            anchor_y='top',
            color=arcade.color.WHITE)

        if self.game_over is True:
            arcade.draw_text(
                text='GAME OVER',
                start_x=LARG_TELA/2, start_y=ALT_TELA/2,
                color=arcade.color.RED, font_size=60,
                font_name='Courier New',
                bold=True,
                align="center", width=LARG_TELA,
                anchor_x="center",
                anchor_y="center")

        if self.pausado is True:
            arcade.draw_text(
                text='PAUSE',
                start_x=LARG_TELA/2, start_y=ALT_TELA/2,
                color=arcade.color.GREEN, font_size=60,
                font_name='Courier New',
                bold=True,
                align="center", width=LARG_TELA,
                anchor_x="center",
                anchor_y="center")


    def on_update(self, delta_time):
        if self.pausado is True:
            return
        self.estrela_list.update()
        if self.game_over is True:
            return

        if self.revive != 0:
            self.revive += 1
            if self.revive >= DT_REVIVE:
                self.revive = 0
                self.nave_list[0].alpha = 255

        if self.bonus_ufo != 0:
            self.bonus_ufo += 1
            if self.bonus_ufo >= DT_UFO:
                self.bonus_ufo = 0

        self.nave_list.update()
        self.vida_list.update()
        self.fase_list.update()
        self.missil_list.update()
        self.inimigo_list.update()
        self.inimissil_list.update()
        self.ufo_list.update()

        # verifica os misseis
        for missil in self.missil_list:
            # verifica se missil acertou inimigo
            acertou_list =\
                arcade.check_for_collision_with_list(
                    missil, self.inimigo_list)
            # se acertou, remove o missil da lista
            if len(acertou_list) > 0:
                missil.remove_from_sprite_lists()
            # atualiza o placar e remove o inimigo
            for inimigo in acertou_list:
                inimigo.remove_from_sprite_lists()
                self.placar += 1

            # verifica se missil acertou inimissil
            acertou_list =\
                arcade.check_for_collision_with_list(
                    missil, self.inimissil_list)
            # se acertou, remove o missil da lista
            if len(acertou_list) > 0:
                missil.remove_from_sprite_lists()
            # remove o inimissil
            for inimissil in acertou_list:
                inimissil.remove_from_sprite_lists()

            # verifica se missil acertou ufo
            acertou_list =\
                arcade.check_for_collision_with_list(
                    missil, self.ufo_list)
            # se acertou, remove o missil da lista
            if len(acertou_list) > 0:
                missil.remove_from_sprite_lists()
                self.bonus_ufo = 1
            # remove o ufo
            for ufo in acertou_list:
                ufo.remove_from_sprite_lists()
            
            # se saiu da tela, remove da lista
            if missil.bottom > ALT_TELA:
                missil.remove_from_sprite_lists()

        # verifica os inimisseis
        for inimissil in self.inimissil_list:
            # verifica se acertou a nave
            acertou_list =\
                arcade.check_for_collision_with_list(
                    inimissil, self.nave_list)
            if self.revive == 0 and\
                len(acertou_list) > 0:
                inimissil.remove_from_sprite_lists()
                if len(self.vida_list) != 0:
                    self.revive = 1
                    self.nave_list[0].alpha = 64
                    vida = self.vida_list.pop()
                    vida.remove_from_sprite_lists()
                else:
                    self.game_over = True

            # se saiu da tela, remove da lista
            if inimissil.top < 0:
                inimissil.remove_from_sprite_lists()

        # verifica se inimigo trombou com a nave
        acertou_list =\
            arcade.check_for_collision_with_list(
                self.nave_list[0], self.inimigo_list)
        if len(acertou_list) > 0:
            self.game_over = True

        # movimenta os inimigos, se encontrou a borda
        # vertical, alterna o sentido do movimento,
        # acelera na horizontal e desce
        x_min = LARG_TELA
        x_max = 0
        for inimigo in self.inimigo_list:
            x_min = min(x_min, inimigo.left)
            x_max = max(x_max, inimigo.right)
        if x_min < 0 or x_max > LARG_TELA:
            for inimigo in self.inimigo_list:
                inimigo.change_x +=\
                    math.copysign(A_X_INIMIGO,\
                        inimigo.change_x)
                inimigo.change_x = -inimigo.change_x
                inimigo.center_y -= V_Y_INIMIGO

        # de vez em quando inimigo lanca inimissil
        for inimigo in self.inimigo_list:
            if random.randrange(P_INIMISSIL) == 0:
                path_sprites = 'spaceshooter/PNG/'\
                    'Lasers/' 
                inimissil = InimissilSprite(
                    path_sprites +
                    'laserGreen04.png',
                    inimigo)
                self.inimissil_list.append(inimissil)

        # de vez em quando cria um ufo
        if random.randrange(P_UFO) == 0:
            path_sprites = 'spaceshooter/PNG/'
            ufo = arcade.Sprite(
                path_sprites + 'ufoBlue.png',
                ESCALA_UFO)
            ufo.change_x = V_X_UFO
            ufo.top = ALT_TELA - 5
            self.ufo_list.append(ufo)

        # verifica de ufo saiu da tela        
        for ufo in self.ufo_list:
            if ufo.left >= LARG_TELA:
                ufo.remove_from_sprite_lists()

        if len(self.inimigo_list) == 0:
            self.fase += 1
            self.inicia_jogo()

    def on_key_press(self, key, modifiers):
        nave = self.nave_list[0]
        if key == arcade.key.P:
            self.pausado = not self.pausado
        elif key == arcade.key.LEFT:
            nave.change_x = -V_X_NAVE
        elif key == arcade.key.RIGHT:
            nave.change_x = V_X_NAVE
        elif key == arcade.key.SPACE:
            if self.game_over is True:
                self.placar = 0
                self.fase = 1
                self.inicia_jogo()
            else:
                # com bonus missil vira metralhadora
                # sem bonus, um missil por vez
                if self.bonus_ufo != 0 or\
                    len(self.missil_list) == 0:
                    path_sprites =\
                        'spaceshooter/PNG/Lasers/' 
                    missil = MissilSprite(
                        path_sprites +
                        'laserRed01.png',
                        nave)
                    self.missil_list.append(missil)

    def on_key_release(self, key, modifiers):
        nave = self.nave_list[0]
        if key == arcade.key.LEFT or\
            key == arcade.key.RIGHT:
            nave.change_x = 0


def main():
    window = MeuJogo()
    window.incia_bg()
    window.inicia_jogo()
    arcade.run()

if __name__ == "__main__":
    main()
