
"""
Jogo de Memoria
Arte proveniente do http://kenney.nl
https://kenney.nl/assets/animal-pack-redux
"""

import random
import arcade
import os

# constantes
ESCALA_CARTA_F = 0.4
ESCALA_CARTA_V = 0.7

MAX_ESPERA = 50

LARG_TELA = 800
ALT_TELA = 600
TIT_TELA = 'Memoria'

FASE_M = [(2,2), (4,4), (6,6), (7,8)]

class CartaSprite(arcade.Sprite):
    def __init__(self, filename, x, y):
        self.nome = filename
        # chama o construtor da superclasse
        super().__init__()
        # carrega 1a. textura, frente da carta
        # primeiro carrega a textura,
        # depois troca a escala
        texture = arcade.load_texture(filename)
        self.scale = ESCALA_CARTA_F
        self.textures.append(texture)
        # carrega 2a. textura, verso da carta
        name = 'platformer-pack-redux-360-assets'\
            '/PNG/Items/coinGold.png'
        texture = arcade.load_texture(name)
        self.scale = ESCALA_CARTA_V
        self.textures.append(texture)
        # posiciona carta na tela
        self.center_x = x
        self.center_y = y
        # inicialmente carta esta fechada
        # 2a. textura
        self.texture = self.textures[1]

    def abre(self):
        # troca a textura para a frente da carta
        # primeiro carrega a textura,
        # depois troca a escala
        self.texture = self.textures[0]
        self.scale = ESCALA_CARTA_F

    def fecha(self):
        # troca a textura para o verso da carta
        # primeiro carrega a textura,
        # depois troca a escala
        self.texture = self.textures[1]
        self.scale = ESCALA_CARTA_V

    def aberta(self):
        # carta esta aberta?    
        return self.texture == self.textures[0]

class MeuJogo(arcade.Window):    
    def __init__(self):
        # muda o diretorio corrente para o mesmo
        # diretorio que esta o fonte .py
        file_path = os.path.dirname(
            os.path.abspath(__file__))
        os.chdir(file_path)

        # determina as dimensoes do maior sprite para
        # criar o baralho na tela
        self.pw_max = 0
        self.ht_max = 0
        self.carta_path = [] 
        path_sprites = 'kenney_animalpackredux/'\
            'PNG/Round (outline)/'
        ldir = os.listdir(path_sprites)
        for arq in ldir:
            arq = arq.upper()
            if arq.find('.PNG') != -1:
                full_path = path_sprites + arq
                carta = arcade.Sprite(full_path,
                    ESCALA_CARTA_F)
                self.pw_max = max(self.pw_max,
                    carta.width)
                self.ht_max = max(self.ht_max,
                    carta.height)
                self.carta_path.append(full_path)
                
        # inicializa a superclasse
        super().__init__(
            LARG_TELA, ALT_TELA,
            TIT_TELA)

        # printa o fundo da tela
        arcade.set_background_color(
            arcade.color.BANGLADESH_GREEN)

        # lista dos sprites
        self.carta_list = None
        self.tenta_list = None
        # flags numericas
        self.acertou = 0
        self.errou = 0
        self.fase = 0
        self.dica = 0
        
    def inicia_jogo(self):
        self.tenta_list = []
        self.acertou = 0
        self.errou = 0
        self.dica = 0
        # colunas e linhas da fase corrente
        qtd_col = FASE_M[self.fase][0]
        qtd_lin = FASE_M[self.fase][1]
        # monta o baralho
        carta_path = self.carta_path.copy()
        selec_path = []
        for i in range(qtd_col*qtd_lin//2):
            path = random.choice(carta_path)
            selec_path.append(path)
            selec_path.append(path)
            carta_path.remove(path)
        # pega cartas do baralho aleatoriamente
        self.carta_list = arcade.SpriteList()
        y = (ALT_TELA - (qtd_lin-1)*self.ht_max)/2
        for j in range(qtd_lin):
            x = (LARG_TELA -\
                (qtd_col-1)*self.pw_max)/2
            for i in range(qtd_col):
                path = random.choice(selec_path)
                carta = CartaSprite(path, x, y)
                x += self.pw_max
                self.carta_list.append(carta)
                selec_path.remove(path)
            y += self.ht_max
   
    def on_draw(self):
        arcade.start_render()
        self.carta_list.draw()

    def on_update(self, delta_time):
        if self.dica != 0:
            self.dica += 1
            if self.dica > MAX_ESPERA:
                self.dica = 0
                self.tenta_list.clear()
                for carta in self.carta_list:
                    carta.fecha()
        elif self.acertou != 0:
            self.acertou += 1
            if self.acertou > MAX_ESPERA:
                self.acertou = 0
                self.tenta_list[0].remove_from_sprite_lists()
                self.tenta_list[1].remove_from_sprite_lists()
                self.tenta_list.clear()
        elif self.errou != 0:
            self.errou += 1
            if self.errou > MAX_ESPERA:
                self.errou = 0
                self.tenta_list[0].fecha()
                self.tenta_list[1].fecha()
                self.tenta_list.clear()

        if len(self.carta_list) == 0:
            self.fase += 1
            if self.fase >= len(FASE_M):
                self.fase = 0
            self.inicia_jogo()
    
    def on_mouse_press(self, x, y,\
            button, modifiers):
        # nao deixa clicar enquanto esta esperando
        if self.acertou != 0 or\
            self.errou != 0 or\
            self.dica != 0:
            return

        if button == arcade.MOUSE_BUTTON_LEFT:
            cartas = arcade.get_sprites_at_point(
                (x,y), self.carta_list)
            if len(cartas) > 0:
                if cartas[0].aberta() is False:
                    cartas[0].abre()
                    self.tenta_list.append(cartas[0])

        if len(self.tenta_list) == 2:
            if self.tenta_list[0].nome ==\
                self.tenta_list[1].nome:
                self.acertou = 1
            else:
                self.errou = 1

    def on_key_press(self, key, modifiers):
        if key == arcade.key.D:
            self.dica = 1
            for carta in self.carta_list:
                carta.abre()

def main():
    window = MeuJogo()
    window.inicia_jogo()
    arcade.run()

if __name__ == "__main__":
    main()
