ABERTA     = 0
FECHADA    = 1
TRAVADA    = 2
DESTRAVADA = 3

class Veiculo():
    def __init__(self, modelo, marca,
                 ano_fabricacao, cor,
                 qtd_rodas, qtd_lugares,
                 tipo_combustivel):
        self.modelo = modelo
        self.marca = marca
        self.ano_fabricacao = ano_fabricacao
        self.cor = cor
        self.qtd_rodas = qtd_rodas
        self.qtd_lugares = qtd_lugares
        self.tipo_combustivel = tipo_combustivel

        self.velocidade = 0
        self.ligado = False

    def liga(self):
        # so liga se estiver desligado
        if self.ligado == False:
            self.ligado = True

    def desliga(self):
        # so desliga se estiver ligado
        if self.ligado == True:
            self.ligado = False

    def acelera(self, aceleracao):
        # nao da pra acelerar veiculo desligado
        if self.ligado == False:
            return
        else:
            self.velocidade += aceleracao

    def freia(self, aceleracao):
        # nao da pra acelerar veiculo desligado
        if self.ligado == False:
            return
        else:
            self.velocidade -= aceleracao

    def vira_direita(self):
        print('->')

    def vira_esquerda(self):
        print('<-')

    def __str__(self):
        str_out = ''
        str_out += 'Modelo      : '\
                + self.modelo + '\n'
        str_out += 'Marca       : '\
                + self.marca + '\n'
        str_out += 'Ano         : '\
                + str(self.ano_fabricacao) + '\n'
        str_out += 'Cor         : '\
                + self.cor + '\n'
        str_out += 'Rodas       : '\
                + str(self.qtd_rodas) + '\n'
        str_out += 'Lugares     : '\
                + str(self.qtd_lugares) + '\n'
        str_comb = ''
        if len(self.tipo_combustivel) > 1:
            str_comb = 'Flex ('
            for comb in self.tipo_combustivel:
                str_comb += comb + ', '
            str_comb = str_comb[:-2]
            str_comb += ')'
        else:
            str_comb = self.tipo_combustivel[0]
        str_out += 'Combustivel : ' + str_comb
        return str_out
    
class CarroPasseio(Veiculo):
    def __init__(self, modelo, marca,
                 ano_fabricacao, cor,
                 qtd_rodas, qtd_lugares,
                 tipo_combustivel,
                 qtd_portas,
                 ar_condic, radio):
        Veiculo.__init__(self, modelo, marca,
                 ano_fabricacao, cor,
                 qtd_rodas, qtd_lugares,
                 tipo_combustivel)

        self.qtd_portas = qtd_portas
        self.ar_condic = ar_condic
        self.radio = radio
        self.portas = [FECHADA] * qtd_portas
    
    def abre_porta(self, porta):
        if porta < 0 or \
           porta > self.qtd_portas - 1:
            raise ValueError('Porta invalida')
        else:
            self.portas[porta] = ABERTA
            
    def fecha_porta(self, porta):
        if porta < 0 or \
           porta > self.qtd_portas - 1:
            raise ValueError('Porta invalida')
        else:
            self.portas[porta] = FECHADA

    def trava_porta(self, porta):
        if porta < 0 or \
           porta > self.qtd_portas - 1:
            raise ValueError('Porta invalida')
        else:
            self.portas[porta] = TRAVADA

    def destrava_porta(self, porta):
        if porta < 0 or \
           porta > self.qtd_portas - 1:
            raise ValueError('Porta invalida')
        else:
            self.portas[porta] = DESTRAVADA

    def __str__(self):
        str_out =  Veiculo.__str__(self) + '\n'
        str_out += 'Portas      : '\
                + str(self.qtd_portas) + '\n'
        str_out += 'A/C         : '\
                + str(self.ar_condic) + '\n'
        str_out += 'Radio       : '\
                + str(self.radio) + '\n'
        return str_out

class CarroCorrida(Veiculo):
    def __init__(self, modelo, marca,
                 ano_fabricacao, cor,
                 aerofolio, suspensao_ativa,
                 controle_tracao):
        Veiculo.__init__(self, modelo, marca,
                 ano_fabricacao, cor,
                 4, 1, ['gasolina'])

        self.aerofolio = aerofolio
        self.suspensao_ativa = suspensao_ativa
        self.controle_tracao = controle_tracao

    def ajusta_aerofolio():
        pass

    def mapeamento_motor():
        pass

    def veloc_max(vel):
        if self.ligado == True:
            self.velocidade = vel

    def __str__(self):
        str_out =  Veiculo.__str__(self) + '\n'
        str_out += 'Aerofolio   : '\
                + str(self.aerofolio) + '\n'
        str_out += 'Suspensao   : '\
                + str(self.suspensao_ativa) + '\n'
        str_out += 'Tracao      : '\
                + str(self.controle_tracao) + '\n'
        return str_out

# entrada do programa
palio = CarroPasseio('Palio', 'Fiat',
            2009, 'azul',
            4, 4,
            ['gasolina'],
            2,
            False, True)
print(palio)
print('Portas ' + str(palio.portas))
palio.abre_porta(1)
print('Portas ' + str(palio.portas))
print()
reneg = CarroPasseio('Renegade', 'Jeep',
              2016, 'prata',
              4, 4,
              ['etanol', 'gasolina'],
              4,
              True, True)
print(reneg)
reneg.liga()
reneg.acelera(1)
print('Velocidade: ' + str(reneg.velocidade))
reneg.vira_direita()
reneg.freia(1)
print('Velocidade: ' + str(reneg.velocidade))
reneg.desliga()
print()
ferrari = CarroCorrida('SF90', 'Ferrari',
        2019, 'vermelho',
        True, True, True)
print(ferrari)
