
"""
Arte proveniente do autor :)
"""

import arcade
import random
import math
import os

# aceleracao da gravidade do jogo
AG = -0.5
# angulos de saida e velocidades da pipoca
ANG_MIN = 25
ANG_MAX = 85
VEL_MIN = 15
VEL_MAX = 25

# tempo total de jogo e quando entra na pressa
T_TOTAL = 90
T_PRESSA = 25

# chance de aparecer uma pipoca
CHANCE_PIPOCA = 4
# distribuicao dos tipos de pipoca, por fase:
# pirua 2/13, pequena 5/13, grande 5/13 e doce 1/13 
# pirua 1/1
# doce 1/1
T_PIPOCA = [[0]*2 + [1]*5 + [2]*5 + [3],
            [0],
            [0]*2 + [1]*5 + [2]*5 + [3],
            [3]]
# tempo, em s, de cada fase
T_FASE = [15,
           3,
          20,
           5] 
# pontos por tipo de pipoca
P_PIPOCA = [ 0, 1, 1, 2]
# bonus de tempo, em s, por tipo de pipoca
B_PIPOCA = [-1, 0, 0, 1]

# consts dos sprites
ESCALA_PIPOCA = 0.25
SPIN_PIPOCA = 5
ESCALA_SAQUINHO = 0.25
ESCALA_PONTOFADE = 0.25
D_ALPHA_PONTOFADE = 3
V_X_SAQUINHO = 5

LARG_TELA = 800
ALT_TELA = 600
TIT_TELA = 'Pipoca'

class PipocaSprite(arcade.Sprite):
    def __init__(self, filename, tipo):
        # calcula escala para mostrar pipocas
        # de tamanhos diferentes
        escala = random.randint(6, 10)/10
        escala *= ESCALA_PIPOCA
        # chama o construtor da superclasse
        super().__init__(filename)
        self.scale = escala
        # posicao inicial da pipoca
        self.center_x = 0
        self.center_y = 0
        # angulo de desenho da pipoca
        self.angle = random.randint(0,360)
        # angulo e velocidade de saida 
        ang = random.randint(ANG_MIN, ANG_MAX)
        vel = random.randint(VEL_MIN, VEL_MAX)
        self.change_x = vel*math.cos(math.pi*ang/180)
        self.change_y = vel*math.sin(math.pi*ang/180)
        self.v0y = self.change_y
        # tipo da pipoca
        self.tipo = tipo

    def update(self):
        # vy = v0y + gt
        self.change_y += AG
        self.angle -= SPIN_PIPOCA
        # chama a superclasse
        super().update()

class SaquinhoSprite(arcade.Sprite):
    def __init__(self, filename):
        # chama o construtor da superclasse
        super().__init__(filename)
        # determinar o novo hitbox ANTES
        # de trocar a escala
        pw = self.width
        ht = self.height
        # hitbox eh a tampa do saquinho, ou seja,
        # nao pega pipocas pela lateral
        points = [
            [-pw/2, ht/2],
            [-pw/2, ht/2-ht/4],
            [pw/2, ht/2-ht/4],
            [pw/2, ht/2]
        ] 
        self.set_hit_box(points)
        self.scale = ESCALA_SAQUINHO
        # posiciona o sprite DEPOIS
        # de trocar a escala
        self.center_x = LARG_TELA / 2
        self.center_y = self.height * 3 / 4

    def update(self):
        # nao deixa sair da tela
        if self.left < 0:
            self.left = 0
        elif self.right > LARG_TELA - 1:
            self.right = LARG_TELA - 1
        # chama a superclasse
        super().update()

class PontoFade(arcade.Sprite):
    def __init__(self, filename, x0, y0, vx=0, vy=0):
        # chama o construtor da superclasse
        super().__init__(filename)
        self.scale = ESCALA_PONTOFADE
        # posicao inicial do ponto fade
        self.center_x = x0
        self.center_y = y0
        self.change_x = vx
        self.change_y = vy

    def update(self):
        # ajusta o alpha
        alpha = self.alpha - D_ALPHA_PONTOFADE
        if alpha < 0:
            alpha = 0
        self.alpha = alpha
        # chama a superclasse
        super().update()

class MeuJogo(arcade.Window):    
    def __init__(self):
        # inicializa a superclasse
        super().__init__(
            LARG_TELA, ALT_TELA,
            TIT_TELA)
        # muda o diretorio corrente para o mesmo
        # diretorio que esta o fonte .py
        file_path = os.path.dirname(
            os.path.abspath(__file__))
        os.chdir(file_path)

        # tempo restante de jogo e demais flags
        self.t_rest = T_TOTAL
        self.placar = 0
        self.recorde = 0
        self.game_over = False
        self.pausado = False
        self.fase = 0
        self.t_fase = T_FASE[self.fase]

        # lista dos sprites
        self.pip_list = None
        self.saq_list = None
        self.ptf_list = None

        # esconde o cursor do mouse
        self.set_mouse_visible(False)

        # printa o fundo da tela
        arcade.set_background_color(
            arcade.color.BABY_BLUE_EYES)

        # path dos sprites das pipocas e ptfs
        self.path_pip = []
        self.path_ptf = []
        for i in range(4):
            self.path_pip.append(
                f'sprites_pipoca/pip{i}.png')
            self.path_ptf.append(
                f'sprites_pipoca/ptf{i}.png')

        # arquivo de recorde
        self.path_rec = 'pipoca.txt'

    def grava_recorde(self):
        with open(self.path_rec, 'wt') as f_arq:
            f_arq.write(str(self.recorde) + '\n')

    def le_recorde(self):
        rec = 0
        with open(self.path_rec, 'rt') as f_arq:
            rec = f_arq.readline()
            rec = rec.strip()
        return int(rec)

    def inicia_jogo(self):
        # se o arquivo de recorde existir, le
        if os.access(self.path_rec, os.F_OK) is True:
            rec = self.le_recorde()
            # verifica se o recorde gravado e maior
            # que o atual
            if rec > self.recorde:
                self.recorde = rec

        # flags
        self.t_rest = T_TOTAL
        self.placar = 0
        self.game_over = False
        self.pausado = False
        self.fase = 0
        self.t_fase = T_FASE[self.fase]

        # lista dos sprites
        self.pip_list = arcade.SpriteList()
        self.saq_list = arcade.SpriteList()
        self.ptf_list = arcade.SpriteList()

        # cria o saquinho
        path = 'sprites_pipoca/saq.png' 
        saq = SaquinhoSprite(path)
        self.saq_list.append(saq)      

    def on_draw(self):
        arcade.start_render()        
        # desenha sprites
        self.pip_list.draw()
        self.saq_list.draw()
        self.ptf_list.draw()

        # tempo de jogo
        cor = arcade.color.MEDIUM_SLATE_BLUE
        if self.t_rest < T_PRESSA:
            cor = arcade.color.RED 
        ang = self.t_rest / T_TOTAL * 360

        raio = 60
        x = LARG_TELA - 5 - raio/2
        y = ALT_TELA -  5 - raio/2
        arcade.draw_arc_filled(x, y,
                #raio, raio,
                raio, raio,
                cor, 90,
                ang + 90)

        # tempo restante
        arcade.draw_text(
            text='%d' % self.t_rest,
            start_x=x, start_y=y,
            color=arcade.color.CORNSILK,
            font_name='Courier New',
            font_size=30,
            bold=True,
            anchor_x='center',
            anchor_y='center')

        # placar
        arcade.draw_text(
            text='PLACAR',
            start_x=LARG_TELA/2, start_y=ALT_TELA - 6,
            color=arcade.color.AO,
            font_name='Courier New',
            font_size=12,
            bold=True,
            anchor_x='center',
            anchor_y='center')
        arcade.draw_text(
            text=str(self.placar),
            start_x=LARG_TELA/2, start_y=y,
            color=arcade.color.AO,
            font_name ='Courier New',
            font_size=30,
            bold = True,
            anchor_x='center',
            anchor_y='center')
        # recorde
        arcade.draw_text(
            text='RECORDE',
            start_x=0, start_y=ALT_TELA - 6,
            color=arcade.color.CRIMSON,
            font_name='Courier New',
            font_size=12,
            anchor_x='left',
            anchor_y='center')
        arcade.draw_text(
            text=str(self.recorde),
            start_x=0, start_y=y,
            color=arcade.color.CRIMSON,
            font_name='Courier New',
            font_size=30,
            bold=True,
            anchor_x='left',
            anchor_y='center')

        if self.pausado is True:
            arcade.draw_text(
                text='PAUSE',
                start_x=LARG_TELA/2, start_y=ALT_TELA/2,
                color=arcade.color.GREEN,
                font_name='Courier New',
                font_size=60,
                bold=True,
                align='center', width=LARG_TELA,
                anchor_x='center',
                anchor_y='center')

        if self.game_over is True:
            arcade.draw_text(
                text='GAME OVER',
                start_x=LARG_TELA/2, start_y=ALT_TELA/2,
                color=arcade.color.RED,
                font_name='Courier New',
                font_size=60,
                bold=True,
                align='center', width=LARG_TELA,
                anchor_x='center',
                anchor_y='center')
            # acabou, grava o recorde
            self.grava_recorde()

        '''
        hit_boxes = True
        if hit_boxes is True:
            for saq in self.saq_list:
                saq.draw_hit_box(arcade.color.RED, 2)
            for pip in self.pip_list:
                pip.draw_hit_box(arcade.color.RED, 2)
        '''

    def on_update(self, delta_time):
        if self.pausado is True:
            return
        if self.game_over is True:
            return
        # atualiza tempo de jogo
        self.t_rest -= delta_time
        if self.t_rest <= 0:
            self.game_over = True

        # atualiza sprites
        self.pip_list.update()
        self.saq_list.update()
        self.ptf_list.update()

        # verifica quais pipocas o saquinho pegou
        pegou_list =\
            arcade.check_for_collision_with_list(
                self.saq_list[0],
                self.pip_list)
        for pip in pegou_list:
            # cria ptf no centro da pipoca, com vx e
            # -vy, ou seja, rebate no saquinho
            ptf = PontoFade(self.path_ptf[pip.tipo],
                        pip.center_x, pip.center_y,
                        pip.change_x, -pip.change_y)
            self.ptf_list.append(ptf)
            # atualiza tempo de jogo e placar
            # conforme tipo da pipoca
            self.t_rest += B_PIPOCA[pip.tipo]
            self.placar += P_PIPOCA[pip.tipo]
            # some com a pipoca
            pip.remove_from_sprite_lists()

        # verifica se a pipoca saiu da tela
        for pip in self.pip_list:
            if pip.center_x > LARG_TELA or\
                    pip.center_x < 0 or\
                    pip.center_y > ALT_TELA or\
                    pip.center_y < 0:
                pip.remove_from_sprite_lists()

        # apaga os ptf que estiverem transparente
        for ptf in self.ptf_list:
            if ptf.alpha == 0:
                ptf.remove_from_sprite_lists()

        if self.t_fase < 0:
            self.fase += 1
            self.fase %= len(T_FASE)
            self.t_fase = T_FASE[self.fase]
        else:
            self.t_fase -= delta_time

        # jogador bateu o recorde?
        if self.placar > self.recorde:
            self.recorde = self.placar

        t_pip = T_PIPOCA[self.fase]
        # de vez em quando cria um pipoca
        if random.randint(1, CHANCE_PIPOCA) == 1:
            t = random.choice(t_pip)
            pip = PipocaSprite(self.path_pip[t], t)
            self.pip_list.append(pip)

    def on_key_press(self, key, modifiers):
        saq = self.saq_list[0]
        if key == arcade.key.P:
            self.pausado = not self.pausado
        elif key == arcade.key.LEFT:
            saq.change_x = -V_X_SAQUINHO
        elif key == arcade.key.RIGHT:
            saq.change_x = V_X_SAQUINHO
        elif key == arcade.key.SPACE:
            if self.game_over is True:
                self.inicia_jogo()

    def on_key_release(self, key, modifiers):
        saq = self.saq_list[0]
        if key == arcade.key.LEFT or\
            key == arcade.key.RIGHT:
            saq.change_x = 0

def main():
    window = MeuJogo()
    window.inicia_jogo()
    arcade.run()

if __name__ == '__main__':
    main()
    
