
"""
Simulador de Dados
"""

import arcade
import random
import os

# constantes
ESCALA_DADO = 1.5
PW_DADO = 120
HT_DADO = 80

MAX_ESPERA = 15

LARG_TELA = 400
ALT_TELA = 300
TIT_TELA = 'Dados'
        
class DadoSprite(arcade.Sprite):
    def __init__(self, x, y):
        super().__init__()
        self.scale = ESCALA_DADO
        # carrega as texturas das faces do dado
        for i in range(6):
            filename = 'boardgamepack/PNG/Dice/' +\
                'dieWhite_border' + str(i+1) + '.png'
            texture = arcade.load_texture(filename)
            self.textures.append(texture)

        # face inicial do dado
        self.face = 1
        # posiciona carta na tela
        self.center_x = x
        self.center_y = y
        self.set_texture(self.face - 1)

    def mostra_face(self, face):
        self.face = face
        self.set_texture(self.face - 1)

class Simul(arcade.Window):    
    def __init__(self):
        super().__init__(
            LARG_TELA, ALT_TELA,
            TIT_TELA)
        # muda o diretorio corrente para o mesmo
        # diretorio que esta o fonte .py
        file_path = os.path.dirname(
            os.path.abspath(__file__))
        os.chdir(file_path)

        # printa o fundo da tela
        arcade.set_background_color(
            arcade.color.DARK_BLUE)
        # 5 FPS
        self.set_update_rate(1/5)

        # tentativas, eventos e probabilidade
        self.tent = 0
        self.evnt = 0
        self.prob = 0

        # dados rolando?
        self.rolando = 0
        # lista dos sprites
        self.dado_list = arcade.SpriteList()
        x = (LARG_TELA - PW_DADO)/2
        y = (ALT_TELA - HT_DADO)/2
        for i in range(2):
            self.dado_list.append(DadoSprite(x,y))
            x += PW_DADO

    def on_draw(self):
        arcade.start_render()
        self.dado_list.draw()

        txt = 'Par e Par: %d'  % self.evnt + '\n' +\
            'Tentativas: %d' % self.tent + '\n' +\
            'Prob: %.2f' % self.prob
            
        arcade.draw_text(
            text=str(txt),
            start_x=5, start_y=ALT_TELA - 10,
            font_name='Courier', font_size=20,
            anchor_y='top',
            color=arcade.color.WHITE,
            multiline=True,
            width=LARG_TELA)
        
    def on_update(self, delta_time):
        # se os dados nao estao rolando,
        if self.rolando == 0:
            return
        self.rolando += 1
        if self.rolando > MAX_ESPERA:
            self.rolando = 0
            self.tent += 1
            # par e par?
            fA = self.dado_list[0].face
            fB = self.dado_list[1].face
            if fA % 2 == 0 and fB % 2 == 0:
                self.evnt += 1
            self.prob = self.evnt / self.tent
        else:
            fA = random.randint(1,6)
            fB = random.randint(1,6)
            self.dado_list[0].mostra_face(fA)
            self.dado_list[1].mostra_face(fB)
                    
    def on_key_press(self, key, modifiers):
        if key == arcade.key.SPACE:
            self.rolando = 1
    
def main():
    window = Simul()
    arcade.run()

if __name__ == "__main__":
    main()
